package jp.ac.takushoku_u.cs;


/**
* ClusterAnalysisクラスはクラスター分析をするためのクラスです。<BR>
* 2つからnまでの対象とし、1からnまでの属性値を元に分析を行います。
* @version 1.2.1
* @author 河原 慎治(Kawahara Shinji)
*/
/**
*最終更新日 2005年12月09日
*/
public class ClusterAnalysis{

/**
* 分析を行うデータマトリクス
*/
    private ArithmeticMatrix dataMatrix;

/**
* UPGMAクラスター化法を表す定数です。	
*/    
    public static final int CLUSTERING_UPGMA = 0;
    
/**
* SLINKクラスター化法を表す定数です。
*/
    public static final int CLUSTERING_SLINK = 1;
    
/**
* CLINKクラスター化法を表す定数です。
*/ 
    public static final int CLUSTERING_CLINK = 2;
    
/**
* 類似度係数を表す定数です。
*/
    public static final int SIMILARITY_COEFFICIENT = 0;
/**
* 非類似度係数を表す定数です。
*/
	public static final int DISSIMILARITY_COEFFICIENT = 1;
	
/**
* マトリクスサイズのデフォルト値
*/
    private static final int CLUSTER_MATRIX_VALUE = 5;

    private static double max;
    	    
    private final static double MAX_VALUE = max;
    
/**
* 分析を行うマトリクスとして要素0の対象を設定します。
*/
    public ClusterAnalysis(){
	ArithmeticMatrix baseMatrix = new ArithmeticMatrix(CLUSTER_MATRIX_VALUE, CLUSTER_MATRIX_VALUE);
	setMatrix(baseMatrix);
    }	

/**
* 対象データを元に分析を行うマトリクスを設定します。<BR>
* @param object オブジェクト数
*/
    public ClusterAnalysis(int object){
    	ArithmeticMatrix baseMatrix = new ArithmeticMatrix(object, CLUSTER_MATRIX_VALUE);
    	setMatrix(baseMatrix);
    }

/**
* 引数を元に分析を行うマトリクスを設定します。<BR>
* @param matrix 分析を行うマトリクス
*/
    public ClusterAnalysis(ArithmeticMatrix matrix){
       
	//データマトリクスを設定する
	setMatrix(matrix);
    }

/**
* 分析を行うマトリクの標準化を行います。
* @return 標準化マトリクス
*/
    public ArithmeticMatrix getStandardizedMatrix(){
    	double[][] element = new double[dataMatrix.getRowLength()][dataMatrix.getColLength()];
    	for(int i=0; i<dataMatrix.getRowLength(); i++){
    		for(int j=0; j<dataMatrix.getColLength(); j++){
    			element[i][j] = getStandard(i, j);
    		}
    	}
    	ArithmeticMatrix mat = new ArithmeticMatrix(element);
    	return mat;
    }
    
/**
* 指定したマトリクスの標準化を行います。
* @param matrix 指定したマトリクス
* @return 標準化マトリクス
*/
    public ArithmeticMatrix getStandardizedMatrix(ArithmeticMatrix matrix){
    	double[][] element = new double[matrix.getRowLength()][matrix.getColLength()];
    	for(int i=0; i<matrix.getRowLength(); i++){
    		for(int j=0; j<matrix.getColLength(); j++){
    			element[i][j] = getStandard(matrix,i,j);
    		}
    	}
    	ArithmeticMatrix mat = new ArithmeticMatrix(element);
    	return mat;
    }

/**
* 非類似度係数または類似度係数を各対象ごとに配置した類似マトリクスを求めます。
* @param RESEMBLANCE_COEFFICIENT 類似係数
* @return 類似マトリクス
*/
    public ArithmeticMatrix getResemblanceMatrix(int RESEMBLANCE_COEFFICIENT){
    	int resem = RESEMBLANCE_COEFFICIENT;
    	ArithmeticMatrix ruiji = new ArithmeticMatrix();
    	if(resem == SIMILARITY_COEFFICIENT){
    		ruiji.matrixCopy(getSimilarityCoefficientMatrix());
    	}
    	if(resem == DISSIMILARITY_COEFFICIENT){
    		ruiji.matrixCopy(getNonResemblanceMatrix());
    	}
    	return ruiji;
    }
    
/**   
* 指定したマトリクスを元に、非類似度係数または類似度係数を各対象ごとに配置した類似マトリクスを求めます。
* @param matrix 指定したマトリクス
* @param RESEMBLANCE_COEFFICIENT 類似係数
* @return 類似マトリクス
*/     
    public ArithmeticMatrix getResemblanceMatrix(ArithmeticMatrix matrix, int RESEMBLANCE_COEFFICIENT){
    	int resem = RESEMBLANCE_COEFFICIENT;
    	ArithmeticMatrix ruiji = new ArithmeticMatrix();
    	if(resem == SIMILARITY_COEFFICIENT){
    		ruiji.matrixCopy(getSimilarityCoefficientMatrix(matrix));
    	}
    	if(resem == DISSIMILARITY_COEFFICIENT){
    		ruiji.matrixCopy(getNonResemblanceMatrix(matrix));
    	}
    	return ruiji;
    }
    
/**
* 類似度係数を各対象の組みについて配置した類似マトリクスを求めます。
* @return 類似マトリクス
*/
    private ArithmeticMatrix getSimilarityCoefficientMatrix(){
    	ArithmeticMatrix mat = new ArithmeticMatrix(dataMatrix.getColLength(),dataMatrix.getColLength());
    	for(int i=0; i<dataMatrix.getColLength(); i++){
    		for(int j=0; j<dataMatrix.getColLength(); j++){
    			if(i!=j){
    				mat.setCell(i,j,getSimilarityCoefficient(i, j));
    				mat.setCell(j,i,getSimilarityCoefficient(i, j));
    			}
    			else{
    				mat.setCell(i,i,0);
    			}
    		}
    	}
    	return mat;
    }
    
/**
* 指定したマトリクスを元に、類似度係数を各対象の組みについて配置した類似マトリクスを求めます。
* @param matrix 指定したマトリクス
* @return 類似マトリクス
*/
    private ArithmeticMatrix getSimilarityCoefficientMatrix(ArithmeticMatrix matrix){
    	ArithmeticMatrix mat = new ArithmeticMatrix(matrix.getColLength(),matrix.getColLength(),matrix.getColNameArray(),matrix.getColNameArray());
        for(int i=0; i<matrix.getColLength(); i++){
        	for(int j=0; j<matrix.getColLength(); j++){
        		if(i!=j){
        			mat.setCell(i,j,getSimilarityCoefficient(matrix, i ,j));
        			mat.setCell(j,i,getSimilarityCoefficient(matrix, i, j));
        		}
        		else{
        			mat.setCell(i, i ,0);
        		}
        	}
        }
        return mat;
        }

/**
* 非類似度係数を各対象の組みについて配置した類似マトリクスを求めます。
* @return 類似マトリクス
*/
    private ArithmeticMatrix getNonResemblanceMatrix(){
    	ArithmeticMatrix mat = new ArithmeticMatrix(dataMatrix.getColLength(),dataMatrix.getColLength());
    	for(int i=0; i<dataMatrix.getColLength(); i++){
    		for(int j=0; j<dataMatrix.getColLength(); j++){
    			if(i!=j){
    				mat.setCell(i,j,getDissimilarityCoefficient(i, j));
    				mat.setCell(j,i,getDissimilarityCoefficient(i, j));
    			}
    			else{
    				mat.setCell(i,i,0);
    			}
    		}
    	}
    	return mat;
    }
    
/**
* 指定したマトリクスを元に、非類似度係数を各対象の組みについて配置した類似マトリクスを求めます。
* @param matrix 指定したマトリクス
* @return 類似マトリクス
*/
    private ArithmeticMatrix getNonResemblanceMatrix(ArithmeticMatrix matrix){
    	int size = matrix.getColLength();
    	ArithmeticMatrix mat = new ArithmeticMatrix(size,size,matrix.getColNameArray(),matrix.getColNameArray());
    	for(int i=0; i<size; i++){
    		for(int j=0; j<size; j++){
    			if(i!=j){
    				mat.setCell(i,j,getDissimilarityCoefficient(matrix, i ,j));
    				mat.setCell(j,i,getDissimilarityCoefficient(matrix, i, j));
    			}
    			else{
    				mat.setCell(i, i ,0);
    			}
    		}
    	}
    	return mat;
    }

/**
* 指定した対象におけるある属性の標準化を行います。
* @param attribute 指定した属性番号
* @param object 指定した対象番号
* @exception NotAttributeException 指定した属性が存在しない場合
* @exception NotObjectException 指定した対象が存在しない場合
* @return 標準化した対象の属性
*/
    public double getStandard(int attribute, int object){
    	if(attribute <= dataMatrix.getRowLength()){
    		if(object <= dataMatrix.getColLength()){
    			ArithmeticMatrix mat = new ArithmeticMatrix();
    			mat.matrixCopy(dataMatrix);
    			double sum = 0;
    			double ave = 0;
    			double hensa = 0;
    			double sth = 0;
    			double data = 0;
    			double sa = 0;
    			double sa2 = 0;
    			/*平均値*/
    			for(int i=0; i<dataMatrix.getColLength(); i++){
    				data = 0;
    				data = mat.getCell(attribute, i);
    				sum = data + sum;
    			}
    			ave = sum/dataMatrix.getColLength();
    			/*標準偏差*/
    			for(int i=0; i<dataMatrix.getColLength(); i++){
    				data = 0;
    				sa = 0;
    				sa2 = 0;
    				data = mat.getCell(attribute, i);
    				sa = data - ave;
    				sa2 = sa * sa;
    				hensa = sa2 + hensa;
    			}
    			sth = Math.sqrt(hensa/(mat.getColLength()-1));
    			
    			return (mat.getCell(attribute, object) - ave)/sth;
    		}
    		else{
    			throw new NotAttributeException("attribute("+ attribute +")");
    		}
    	}
    	else{
    			throw new NotObjectException("object("+ object +")");
    	}
    }
    

/**
* 指定したマトリクスを元に、指定した対象におけるある属性の標準化を行います。
* @param matrix 指定したマトリクス
* @param attribute 指定した属性番号
* @param object 指定した対象番号
* @exception NotAttributeException 指定した属性が存在しない場合
* @exception NotObjectException 指定した対象が存在しない場合
* @return 標準化した対象の属性
*/
    public double getStandard(ArithmeticMatrix matrix, int attribute, int object){
    	if(attribute <= dataMatrix.getRowLength()){
    		if(object <= dataMatrix.getColLength()){
    			ArithmeticMatrix mat = new ArithmeticMatrix();
    			mat.matrixCopy(matrix);
    			double sum = 0;
    			double ave = 0;
    			double hensa = 0;
    			double sth = 0;
    			double data = 0;
    			double sa = 0;
    			double sa2 = 0;
    			/*平均値*/
    			for(int i=0; i<matrix.getColLength(); i++){
    				data = 0;
    				data = mat.getCell(attribute, i);
    				sum = data + sum;
    			}
    			ave = sum/matrix.getColLength();
    			/*標準偏差*/
    			for(int i=0; i<matrix.getColLength(); i++){
    				data = 0;
    				sa = 0;
    				sa2 = 0;
    				data = mat.getCell(attribute, i);
    				sa = data - ave;
    				sa2 = sa * sa;
    				hensa = sa2 + hensa;
    			}
    			sth = Math.sqrt(hensa/(mat.getColLength()-1));
    			
    			return (mat.getCell(attribute, object) - ave)/sth;
    		}
    		else{
    			throw new NotAttributeException("attribute("+ attribute +")");
    		}
    	}
    	else{
    		throw new NotObjectException("object("+ object +")");
    	}
    }
    
/**
* 2つの対象の類似度係数を求めます。
* @param object1 指定した1つ目の対象番号
* @param object2 指定した2つ目の対象番号
* @exception NotObjectException 指定した対象が存在しない場合
* @return 類似度係数 
*/
    public double getSimilarityCoefficient(int object1, int object2){
    	if(object1 <= dataMatrix.getColLength() && object2 <= dataMatrix.getColLength()){
    		double sum1 = 0;
    		double data1 = 0;
    		double data2 = 0;
    		double[] ave = dataMatrix.getRowAverageArray();
    		double sqr1 = 0;
    		double sqr2 = 0;
    		double nume = 0;
    		double deno = 0;
    	
    		for(int i=0; i<dataMatrix.getRowLength(); i++){
    			sum1 = 0;
    			data1 = 0;
    			data2 = 0;
    			sum1 = (dataMatrix.getCell(i,object1)-ave[i]) * (dataMatrix.getCell(i,object2)-ave[i]);
    			nume = sum1 + nume;
    		
    			data1 = (dataMatrix.getCell(i,object1) - ave[i]) * (dataMatrix.getCell(i,object1) - ave[i]);
    			sqr1 = data1 + sqr1;
    		
    			data2 = (dataMatrix.getCell(i,object2) - ave[i]) * (dataMatrix.getCell(i,object2) - ave[i]);
    			sqr2 = data2 + sqr2;
    		}
    		deno = Math.sqrt(sqr1) * Math.sqrt(sqr2);
    	
    		return nume/deno;
    	}
    	else{
    		throw new NotObjectException("object("+ object1 +","+object2+")");
    	}
    }
/**
* 指定されたマトリクスを元に、ある２つの対象の類似度係数を求めます。
* @param matrix 分析を行うマトリクス
* @param object1 指定した1つ目の対象番号
* @param object2 指定した2つ目の対象番号
* @return 類似度係数 
*/
    public double getSimilarityCoefficient(ArithmeticMatrix matrix, int object1, int object2){
    	if(object1 <= dataMatrix.getColLength() && object2 <= dataMatrix.getColLength()){
    		double sum1 = 0;
    		double data1 = 0;
    		double data2 = 0;
    		double[] ave = matrix.getRowAverageArray();
    		double sqr1 = 0;
    		double sqr2 = 0;
    		double nume = 0;
    		double deno = 0;
   
    	
    		for(int i=0; i<matrix.getRowLength(); i++){
    			sum1 = 0;
    			data1 = 0;
    			data2 = 0;
    			sum1 = (matrix.getCell(i,object1)-ave[i]) * (matrix.getCell(i,object2)-ave[i]);
    			nume = sum1 + nume;
    		
    			data1 = (matrix.getCell(i,object1) - ave[i]) * (matrix.getCell(i,object1) - ave[i]);
    			sqr1 = data1 + sqr1;
    			data2 = (matrix.getCell(i,object2) - ave[i]) * (matrix.getCell(i,object2) - ave[i]);
    			sqr2 = data2 + sqr2;
    		}
    		deno = Math.sqrt(sqr1) * Math.sqrt(sqr2);
    	
    		return nume/deno;
    	}
    	else{
    		throw new NotObjectException("object("+ object1 +","+object2+")");
    	}
    }


/**
* 指定した２つの対象の非類似度係数を求めます。
* @param object1 指定した1つ目の対象番号
* @param object2 指定した2つ目の対象番号
* @return 非類似度係数
*/
    public double getDissimilarityCoefficient(int object1, int object2){
    	if(object1 <= dataMatrix.getColLength() && object2 <= dataMatrix.getColLength()){
    		ArithmeticMatrix mat = new ArithmeticMatrix();
    		mat.matrixCopy(dataMatrix);
    		double data1 = 0; 
    		double data2 = 0;
    		double sub = 0;
    		double squ = 0;
    		double sum = 0;
    	
    		for(int i=0; i<dataMatrix.getRowLength(); i++){
    			data1 = 0;
    			data2 = 0;
    			sub = 0;
    			squ = 0;
    			data1 = mat.getCell(i,object1);
    			data2 = mat.getCell(i,object2);
    			sub = data1 - data2;
    			squ = sub * sub;
    			sum = sum + squ;
    		}
    		return Math.sqrt(sum);
    	}
    	else{
    		throw new NotObjectException("object("+ object1 +","+object2+")");
    	}
    }
    	
    /**
    * 指定したマトリクスを元に、ある２つの対象においての非類似度係数を求めます。
    * @param matrix 分析を行うマトリクス
    * @param object1 指定した1つ目の対象番号
    * @param object2 指定した2つ目の対象番号
    * @return 非類似度係数
    */
        public double getDissimilarityCoefficient(ArithmeticMatrix matrix, int object1, int object2){
        	if(object1 <= dataMatrix.getColLength() && object2 <= dataMatrix.getColLength()){
        		ArithmeticMatrix mat = new ArithmeticMatrix();
        		mat.matrixCopy(matrix);
        		double data1 = 0;
        		double data2 = 0;
        		double sub = 0;
        		double squ = 0;
        		double sum = 0;
        	
        		for(int i=0; i<mat.getRowLength(); i++){       		
        			data1 = 0;
        			data2 = 0;
        			sub = 0;
        			squ = 0;
        			data1 = mat.getCell(i,object1);
        			data2 = mat.getCell(i,object2);
        			sub = data1 - data2;
        			squ = sub * sub;
        			sum = sum + squ;
        		}
        		return Math.sqrt(sum);
        	}
        	else{
        		throw new NotObjectException("object("+ object1 +","+object2+")");
        	}
        }
    
/**
* 指定したデータマトリクスを選択されたクラスター化手法と類似係数を元に、コーフェンマトリクスに変換します。 
* @param matrix データマトリクス
* @param CLUSTERING_METHOD クラスター化手法を表す定数
* @param RESEMBLANCE_COEFFICIENT 類似係数を表す定数
* @return コーフェンマトリクス
*/
   public ArithmeticMatrix getCorfeneticMatrix(ArithmeticMatrix matrix, int CLUSTERING_METHOD,int RESEMBLANCE_COEFFICIENT){
   	    int meth = CLUSTERING_METHOD;
   	    int re =RESEMBLANCE_COEFFICIENT;
   	    ArithmeticMatrix corfen = new ArithmeticMatrix();
   	    switch(meth){
		case CLUSTERING_UPGMA:
			if(re == SIMILARITY_COEFFICIENT){
				corfen.matrixCopy(getUpgmaCorfeneticMatrix(matrix,re));	
			}
			if(re == DISSIMILARITY_COEFFICIENT){
				corfen.matrixCopy(getUpgmaCorfeneticMatrix(matrix,re));
			}
			break;
   	    
		case CLUSTERING_SLINK:
			if(re == SIMILARITY_COEFFICIENT){
				corfen.matrixCopy(getSlinkCorfeneticMatrix(matrix,re));
			}
			if(re == DISSIMILARITY_COEFFICIENT){
				corfen.matrixCopy(getSlinkCorfeneticMatrix(matrix,re));
			}
			break;
			
		case CLUSTERING_CLINK:
			if(re == SIMILARITY_COEFFICIENT){
				corfen.matrixCopy(getClinkCorfeneticMatrix(matrix,re));
			}
			if(re == DISSIMILARITY_COEFFICIENT){
				corfen.matrixCopy(getClinkCorfeneticMatrix(matrix,re));
			}
			break;
   	    }	
		return corfen;
   }
   
   /**
   * データマトリクスを選択されたクラスター化手法と類似係数を元に、コーフェンマトリクスに変換します。 
   * @param CLUSTERING_METHOD クラスター化手法を表す定数
   * @param RESEMBLANCE_COEFFICIENT 類似係数を表す定数
   * @return コーフェンマトリクス
   */
      public ArithmeticMatrix getCorfeneticMatrix(int CLUSTERING_METHOD,int RESEMBLANCE_COEFFICIENT){
      	    int meth = CLUSTERING_METHOD;
      	    int re =RESEMBLANCE_COEFFICIENT;
      	    ArithmeticMatrix corfen = new ArithmeticMatrix();
      	    switch(meth){
   		case CLUSTERING_UPGMA:
   			if(re == SIMILARITY_COEFFICIENT){
   				corfen.matrixCopy(getUpgmaCorfeneticMatrix(dataMatrix,re));	
   			}
   			if(re == DISSIMILARITY_COEFFICIENT){
   				corfen.matrixCopy(getUpgmaCorfeneticMatrix(dataMatrix,re));
   			}
   			break;
      	    
   		case CLUSTERING_SLINK:
   			if(re == SIMILARITY_COEFFICIENT){
   				corfen.matrixCopy(getSlinkCorfeneticMatrix(dataMatrix,re));
   			}
   			if(re == DISSIMILARITY_COEFFICIENT){
   				corfen.matrixCopy(getSlinkCorfeneticMatrix(dataMatrix,re));
   			}
   			break;
   			
   		case CLUSTERING_CLINK:
   			if(re == SIMILARITY_COEFFICIENT){
   				corfen.matrixCopy(getClinkCorfeneticMatrix(dataMatrix,re));
   			}
   			if(re == DISSIMILARITY_COEFFICIENT){
   				corfen.matrixCopy(getClinkCorfeneticMatrix(dataMatrix,re));
   			}
   			break;
      	    }	
   		return corfen;
      }
      
    
/**
* データマトリクスをコーフェン係数マトリクスに変換します。
* @param matrix マトリクス
* @param resemcoef 類似係数を表す定数
* @return コーフェンマトリクス
*/
    private ArithmeticMatrix getUpgmaCorfeneticMatrix(ArithmeticMatrix matrix, int resemcoef){
    	int re = resemcoef;
    	if(re == DISSIMILARITY_COEFFICIENT){
    	int size = matrix.getColLength();
    	//類似マトリクスを設定
    	ArithmeticMatrix ruijiMatrix = new ArithmeticMatrix();
    	//類似マトリクスの配列名を決定
    	String[] name = new String[matrix.getColLength()];
    	for(int i=0; i<size; i++){
    		name[i] = String.valueOf(i+",");
    	}
    	ruijiMatrix.matrixCopy(getNonResemblanceMatrix(matrix));
    	ruijiMatrix.nameColCopy(name);
    	ruijiMatrix.nameRowCopy(name);
    	
    		//コーフェンマトリクスを設定
    	ArithmeticMatrix corfen = new ArithmeticMatrix(size,size);
    	corfen.nameColCopy(matrix.getColNameArray());
    	corfen.nameRowCopy(matrix.getColNameArray());
    
    	//再配列用マトリクスの設定
    	ArithmeticMatrix umat = new ArithmeticMatrix();
    	umat.matrixCopy(ruijiMatrix);
    	//最小値	
    	int[] rowcol = minRowCol(umat);
    	int row = rowcol[0];
    	int col = rowcol[1];
    
    	int count = size - 2;  //マトリクスの再配列の回数
    	
    	//1回目の再配列
    	corfen.setCell(row,col,umat.getCell(row,col));
    	corfen.setCell(col,row,umat.getCell(col,row));
    	umat.matrixCopy(getUpgmaObjectUniteMatrix(row,col,ruijiMatrix,umat));
    	//2回目以降のクラスター化
    	String rn;
    	String rname[];
    	String cn;
    	String cname[];
		int rs=0;
		int cs=0;
    	int rc[] = new int[2];
    	if(size>3){
    		for(int i=0; i<count; i++){
    			rc = minRowCol(umat);
    			row = rc[0];
    			rn = umat.getRowName(row);
    			rname = rn.split(",");
    			rs = rname.length;
    			col = rc[1];
    			cn = umat.getColName(col);
    			cname = cn.split(",");
    			cs = cname.length;
    	
    		//	outputDoubleMatrix(umat);
    			for(int j=0; j<rs; j++){
    				for(int k=0; k<cs; k++){
    					corfen.setCell(Integer.parseInt(rname[j]),Integer.parseInt(cname[k]),getMinCell(umat));
    					corfen.setCell(Integer.parseInt(cname[k]),Integer.parseInt(rname[j]),getMinCell(umat));
    				}
    			}
    			umat.matrixCopy(getUpgmaObjectUniteMatrix(row,col,ruijiMatrix,umat));
    	
    		}
    	}
    	
    	return corfen;
    	}
    	if(re == SIMILARITY_COEFFICIENT){
    		int size = matrix.getColLength();
        	//類似マトリクスを設定
        	ArithmeticMatrix ruijiMatrix = new ArithmeticMatrix();
        	//類似マトリクスの配列名を決定
        	String[] name = new String[matrix.getColLength()];
        	for(int i=0; i<size; i++){
        		name[i] = String.valueOf(i+",");
        	}
        	ruijiMatrix.matrixCopy(getSimilarityCoefficientMatrix(matrix));
        	ruijiMatrix.nameColCopy(name);
        	ruijiMatrix.nameRowCopy(name);
        	
        		//コーフェンマトリクスを設定
        	ArithmeticMatrix corfen = new ArithmeticMatrix(size,size);
        	corfen.nameColCopy(matrix.getColNameArray());
        	corfen.nameRowCopy(matrix.getColNameArray());
        
        	//再配列用マトリクスの設定
        	ArithmeticMatrix umat = new ArithmeticMatrix();
        	umat.matrixCopy(ruijiMatrix);
        	//最小値	
        	int[] rowcol = maxRowCol(umat);
        	int row = rowcol[0];
        	int col = rowcol[1];
        
        	int count = size - 2;  //マトリクスの再配列の回数
        	
        	//1回目の再配列
        	corfen.setCell(row,col,umat.getCell(row,col));
        	corfen.setCell(col,row,umat.getCell(col,row));
        	umat.matrixCopy(getUpgmaObjectUniteMatrix(row,col,ruijiMatrix,umat));
        	//2回目以降のクラスター化
        	String rn;
        	String rname[];
        	String cn;
        	String cname[];
    		int rs=0;
    		int cs=0;
        	int rc[] = new int[2];
        	if(size>3){
        		for(int i=0; i<count; i++){
        			rc = maxRowCol(umat);
        			row = rc[0];
        			rn = umat.getRowName(row);
        			rname = rn.split(",");
        			rs = rname.length;
        			col = rc[1];
        			cn = umat.getColName(col);
        			cname = cn.split(",");
        			cs = cname.length;
        	
        		//	outputDoubleMatrix(umat);
        			for(int j=0; j<rs; j++){
        				for(int k=0; k<cs; k++){
        					corfen.setCell(Integer.parseInt(rname[j]),Integer.parseInt(cname[k]),getMaxCell(umat));
        					corfen.setCell(Integer.parseInt(cname[k]),Integer.parseInt(rname[j]),getMaxCell(umat));
        				}
        			}
        			umat.matrixCopy(getUpgmaObjectUniteMatrix(row,col,ruijiMatrix,umat));
        	
        		}
        	}
        	
        	return corfen;
    		
    	}
		return null;
    }


    
    /**
    * データマトリクスをコーフェン係数マトリクスに変換します。
    * @param matrix マトリクス
    * @param resecoef 類似係数を表す定数
    * @return コーフェンマトリクス
    */
        private ArithmeticMatrix getSlinkCorfeneticMatrix(ArithmeticMatrix matrix, int resecoef ){
        	int re = resecoef;
        	if(re == DISSIMILARITY_COEFFICIENT){
        	int size = matrix.getColLength();
        	//類似マトリクスを設定
        	ArithmeticMatrix ruijiMatrix = new ArithmeticMatrix();
        	//類似マトリクスの配列名を決定
        	String[] name = new String[matrix.getColLength()];
        	for(int i=0; i<size; i++){
        		name[i] = String.valueOf(i+",");
        	}
        	ruijiMatrix.matrixCopy(getNonResemblanceMatrix(matrix));
        	ruijiMatrix.nameColCopy(name);
        	ruijiMatrix.nameRowCopy(name);
        	
        		//コーフェンマトリクスを設定
        	ArithmeticMatrix corfen = new ArithmeticMatrix(size,size);
        	corfen.nameColCopy(matrix.getColNameArray());
        	corfen.nameRowCopy(matrix.getColNameArray());
        
        	//再配列用マトリクスの設定
        	ArithmeticMatrix umat = new ArithmeticMatrix();
        	umat.matrixCopy(ruijiMatrix);
        	//最小値	
        	int[] rowcol = minRowCol(umat);
        	int row = rowcol[0];
        	int col = rowcol[1];
        
        	int count = size - 2;  //マトリクスの再配列の回数
        	
        	//1回目の再配列
        	corfen.setCell(row,col,umat.getCell(row,col));
        	corfen.setCell(col,row,umat.getCell(col,row));
        	umat.matrixCopy(getSlinkObjectUniteMatrix(row,col,ruijiMatrix,umat));
        	//2回目以降のクラスター化
        	String rn;
        	String rname[];
        	String cn;
        	String cname[];
    		int rs=0;
    		int cs=0;
        	int rc[] = new int[2];
        	if(size>3){
        		for(int i=0; i<count; i++){
        			rc = minRowCol(umat);
        			row = rc[0];
        			rn = umat.getRowName(row);
        			rname = rn.split(",");
        			rs = rname.length;
        			col = rc[1];
        			cn = umat.getColName(col);
        			cname = cn.split(",");
        			cs = cname.length;
        	
        		//	outputDoubleMatrix(umat);
        			for(int j=0; j<rs; j++){
        				for(int k=0; k<cs; k++){
        					corfen.setCell(Integer.parseInt(rname[j]),Integer.parseInt(cname[k]),getMinCell(umat));
        					corfen.setCell(Integer.parseInt(cname[k]),Integer.parseInt(rname[j]),getMinCell(umat));
        				}
        			}
        			umat.matrixCopy(getSlinkObjectUniteMatrix(row,col,ruijiMatrix,umat));
        	
        		}
        	}
        
        	return corfen;
        	}
        	if(re == SIMILARITY_COEFFICIENT){
        		int size = matrix.getColLength();
            	//類似マトリクスを設定
            	ArithmeticMatrix ruijiMatrix = new ArithmeticMatrix();
            	//類似マトリクスの配列名を決定
            	String[] name = new String[matrix.getColLength()];
            	for(int i=0; i<size; i++){
            		name[i] = String.valueOf(i+",");
            	}
            	ruijiMatrix.matrixCopy(getNonResemblanceMatrix(matrix));
            	ruijiMatrix.nameColCopy(name);
            	ruijiMatrix.nameRowCopy(name);
            	
            		//コーフェンマトリクスを設定
            	ArithmeticMatrix corfen = new ArithmeticMatrix(size,size);
            	corfen.nameColCopy(matrix.getColNameArray());
            	corfen.nameRowCopy(matrix.getColNameArray());
            
            	//再配列用マトリクスの設定
            	ArithmeticMatrix umat = new ArithmeticMatrix();
            	umat.matrixCopy(ruijiMatrix);
            	//最小値	
            	int[] rowcol = maxRowCol(umat);
            	int row = rowcol[0];
            	int col = rowcol[1];
            
            	int count = size - 2;  //マトリクスの再配列の回数
            	
            	//1回目の再配列
            	corfen.setCell(row,col,umat.getCell(row,col));
            	corfen.setCell(col,row,umat.getCell(col,row));
            	umat.matrixCopy(getClinkObjectUniteMatrix(row,col,ruijiMatrix,umat));
            	//2回目以降のクラスター化
            	String rn;
            	String rname[];
            	String cn;
            	String cname[];
        		int rs=0;
        		int cs=0;
            	int rc[] = new int[2];
            	if(size>3){
            		for(int i=0; i<count; i++){
            			rc = maxRowCol(umat);
            			row = rc[0];
            			rn = umat.getRowName(row);
            			rname = rn.split(",");
            			rs = rname.length;
            			col = rc[1];
            			cn = umat.getColName(col);
            			cname = cn.split(",");
            			cs = cname.length;
            	
            		//	outputDoubleMatrix(umat);
            			for(int j=0; j<rs; j++){
            				for(int k=0; k<cs; k++){
            					corfen.setCell(Integer.parseInt(rname[j]),Integer.parseInt(cname[k]),getMaxCell(umat));
            					corfen.setCell(Integer.parseInt(cname[k]),Integer.parseInt(rname[j]),getMaxCell(umat));
            				}
            			}
            			umat.matrixCopy(getClinkObjectUniteMatrix(row,col,ruijiMatrix,umat));
            	
            		}
            	}
            
            	return corfen;
        		
        	}
			return null;
        }
        
        /**
         * データマトリクスをコーフェン係数マトリクスに変換します。
         * @param matrix マトリクス
         * @param resecoef
         * @return コーフェンマトリクス
         */
             private ArithmeticMatrix getClinkCorfeneticMatrix(ArithmeticMatrix matrix,int resecoef){
             	int re = resecoef;
             	if(re == DISSIMILARITY_COEFFICIENT){
             		int size = matrix.getColLength();
             		//類似マトリクスを設定
             		ArithmeticMatrix ruijiMatrix = new ArithmeticMatrix();
             		//類似マトリクスの配列名を決定
             		String[] name = new String[matrix.getColLength()];
             		for(int i=0; i<size; i++){
             			name[i] = String.valueOf(i+",");
             		}
             		ruijiMatrix.matrixCopy(getNonResemblanceMatrix(matrix));
             		ruijiMatrix.nameColCopy(name);
             		ruijiMatrix.nameRowCopy(name);
             	
             			//コーフェンマトリクスを設定
             		ArithmeticMatrix corfen = new ArithmeticMatrix(size,size);
             		corfen.nameColCopy(matrix.getColNameArray());
             		corfen.nameRowCopy(matrix.getColNameArray());
             
             		//再配列用マトリクスの設定
             		ArithmeticMatrix umat = new ArithmeticMatrix();
             		umat.matrixCopy(ruijiMatrix);
             		//最小値	
             		int[] rowcol = minRowCol(umat);
             		int row = rowcol[0];
             		int col = rowcol[1];
             
             		int count = size - 2;  //マトリクスの再配列の回数
             	
             		//1回目の再配列
             		corfen.setCell(row,col,umat.getCell(row,col));
             		corfen.setCell(col,row,umat.getCell(col,row));
             		umat.matrixCopy(getClinkObjectUniteMatrix(row,col,ruijiMatrix,umat));
             		//2回目以降のクラスター化
             		String rn;
             		String rname[];
             		String cn;
             		String cname[];
             		int rs=0;
             		int cs=0;
             		int rc[] = new int[2];
             		if(size>3){
             			for(int i=0; i<count; i++){
             				rc = minRowCol(umat);
             				row = rc[0];
             				rn = umat.getRowName(row);
             				rname = rn.split(",");
             				rs = rname.length;
             				col = rc[1];
             				cn = umat.getColName(col);
             				cname = cn.split(",");
             				cs = cname.length;
             	
             		//	outputDoubleMatrix(umat);
             				for(int j=0; j<rs; j++){
             					for(int k=0; k<cs; k++){
             						corfen.setCell(Integer.parseInt(rname[j]),Integer.parseInt(cname[k]),getMinCell(umat));
             						corfen.setCell(Integer.parseInt(cname[k]),Integer.parseInt(rname[j]),getMinCell(umat));
             					}
             				}
             				umat.matrixCopy(getClinkObjectUniteMatrix(row,col,ruijiMatrix,umat));
             			}
             		}
             
             		return corfen;
             	}
             	
             	if(re == SIMILARITY_COEFFICIENT){
             		int size = matrix.getColLength();
             		//類似マトリクスを設定
             		ArithmeticMatrix ruijiMatrix = new ArithmeticMatrix();
             		//類似マトリクスの配列名を決定
             		String[] name = new String[matrix.getColLength()];
             		for(int i=0; i<size; i++){
             			name[i] = String.valueOf(i+",");
             		}
             		ruijiMatrix.matrixCopy(getNonResemblanceMatrix(matrix));
             		ruijiMatrix.nameColCopy(name);
             		ruijiMatrix.nameRowCopy(name);
                     	
             		//コーフェンマトリクスを設定
             		ArithmeticMatrix corfen = new ArithmeticMatrix(size,size);
             		corfen.nameColCopy(matrix.getColNameArray());
             		corfen.nameRowCopy(matrix.getColNameArray());
             		
             		//再配列用マトリクスの設定
             		ArithmeticMatrix umat = new ArithmeticMatrix();
             		umat.matrixCopy(ruijiMatrix);
             		//最小値	
             		int[] rowcol = maxRowCol(umat);
             		int row = rowcol[0];
             		int col = rowcol[1];
             		
             		int count = size - 2;  //マトリクスの再配列の回数
                     	
             		//1回目の再配列
             		corfen.setCell(row,col,umat.getCell(row,col));
             		corfen.setCell(col,row,umat.getCell(col,row));
             		umat.matrixCopy(getSlinkObjectUniteMatrix(row,col,ruijiMatrix,umat));
             		//2回目以降のクラスター化
             		String rn;
             		String rname[];
             		String cn;
             		String cname[];
             		int rs=0;
             		int cs=0;
             		int rc[] = new int[2];
             		if(size>3){
             			for(int i=0; i<count; i++){
             				rc = maxRowCol(umat);
             				row = rc[0];
             				rn = umat.getRowName(row);
             				rname = rn.split(",");
             				rs = rname.length;
             				col = rc[1];
             				cn = umat.getColName(col);
             				cname = cn.split(",");
             				cs = cname.length;
             				
             				//	outputDoubleMatrix(umat);
             				for(int j=0; j<rs; j++){
             					for(int k=0; k<cs; k++){
             						corfen.setCell(Integer.parseInt(rname[j]),Integer.parseInt(cname[k]),getMaxCell(umat));
             						corfen.setCell(Integer.parseInt(cname[k]),Integer.parseInt(rname[j]),getMaxCell(umat));
             					}
             				}
             				umat.matrixCopy(getSlinkObjectUniteMatrix(row,col,ruijiMatrix,umat));
                     	
             			}
             		}
             		return corfen;
             	}
             	return null;
             }

/**
* 特定の2つの対象の融合を行った時の類似マトリクスを返します。
* @param object1 融合を行う1つ目の対象番号
* @param object2 融合を行う2つ目の対象番号
* @return 融合を行った類似マトリクス 
*/
    private ArithmeticMatrix getUpgmaObjectUniteMatrix(int object1, int object2, ArithmeticMatrix ruijiMatrix,ArithmeticMatrix uniteMatrix){
   	
    	int size = ruijiMatrix.getColLength();
        //融合後のマトリクス
    	int usize = uniteMatrix.getColLength();
    	ArithmeticMatrix umat = new ArithmeticMatrix(usize-1,usize-1);
    	
    	//類似マトリクスを作成
    	ArithmeticMatrix ruimat = new ArithmeticMatrix();
    	ruimat.matrixCopy(ruijiMatrix);
    	//名前を取得する
    	String o1 = uniteMatrix.getColName(object1);
    	String o2 = uniteMatrix.getColName(object2);
    	String st = o1+o2;
    	String str[];
    	str = st.split(",");
    	int si = str.length;
    	
    	int[] num = new int[usize-1];
    	int count=0;
    	//融合させないオブジェクト番号を取得
    	for(int i=0; i<usize; i++){
    		if(i!=object1 && i!=object2){
    			num[count]=i;
    			count++;
    		}
    	}
    	//今回の融合には関係のないデータの配置
    	for(int j=0; j<usize-2; j++){
    		for(int k=0; k<usize-2; k++){
    			umat.setCell(j,k,uniteMatrix.getCell(num[j],num[k]));
    			umat.setCell(k,j,uniteMatrix.getCell(num[k],num[j]));
    		}
    	} 
        double wadata=0;
        int data1;
        String sdata;
        double data2;
        int count2 = count;
        count2 = 0;
        int sri = 0;
        int sd = 0;
        int s = 0;
        //今回の融合に関係のあるデータの配置
        for(int k=0; k<count; k++){
        	for(int i=0; i<si; i++){
        		data1 = Integer.parseInt(str[i]);
        		sdata = uniteMatrix.getColName(num[k]);
        		String sr[] = sdata.split(",");
        		sri = sr.length;
        		for(int j=0; j<sri; j++){
        			sd = Integer.parseInt(sr[j]);
        			data2 = ruimat.getCell(data1,sd); 
        			wadata = wadata + data2;
        			s++;
        		}
        	}
        	wadata = wadata/s;
        	s = 0;
        	//融合に関係あるobjectをマトリクスに設定
     		umat.setCell(usize-2,count2,wadata);
     		umat.setCell(count2,usize-2,wadata);
     		data1 = 0;
     		data2 = 0;
     		wadata = 0;
     		count2++;
        }
        
        //umatの配列名設定
        String[] ar = new String[usize-1];
        int z=0;
        for(int i=0; i<usize; i++){
        	if(i!=object1 && i!=object2){
        		ar[z] = uniteMatrix.getColName(i);
        		z++;
        	}
        }
        ar[usize-2] = st;
        umat.nameColCopy(ar);
        umat.nameRowCopy(ar);
        return umat;
    }

    /**
    * 特定の2つの対象の融合を行った時の類似マトリクスを返します。
    * @param object1 融合を行う1つ目の対象番号
    * @param object2 融合を行う2つ目の対象番号
    * @return 融合を行った類似マトリクス 
    */
        private ArithmeticMatrix getSlinkObjectUniteMatrix(int object1, int object2, ArithmeticMatrix ruijiMatrix,ArithmeticMatrix uniteMatrix){
       	
        	int size = ruijiMatrix.getColLength();
            //融合後のマトリクス
        	int usize = uniteMatrix.getColLength();
        	ArithmeticMatrix umat = new ArithmeticMatrix(usize-1,usize-1);
        	
        	//類似マトリクスを作成
        	ArithmeticMatrix ruimat = new ArithmeticMatrix();
        	ruimat.matrixCopy(ruijiMatrix);
        	//名前を取得する
        	String o1 = uniteMatrix.getColName(object1);
        	String o2 = uniteMatrix.getColName(object2);
        	String st = o1+o2;
        	String str[];
        	str = st.split(",");
        	int si = str.length;
        	
        	int[] num = new int[usize-1];
        	int count=0;
        	//融合させないオブジェクト番号を取得
        	for(int i=0; i<usize; i++){
        		if(i!=object1 && i!=object2){
        			num[count]=i;
        			count++;
        		}
        	}
        	//今回の融合には関係のないデータの配置
        	for(int j=0; j<usize-2; j++){
        		for(int k=0; k<usize-2; k++){
        			umat.setCell(j,k,uniteMatrix.getCell(num[j],num[k]));
        			umat.setCell(k,j,uniteMatrix.getCell(num[k],num[j]));
        		}
        	} 
            int data1;
            String sdata;
            double data2;
            int count2 = count;
            count2 = 0;
            int sri = 0;
            int sd = 0;
            double min = 1000000000;
            //今回の融合に関係のあるデータの配置
            for(int k=0; k<count; k++){
            	for(int i=0; i<si; i++){
            		data1 = Integer.parseInt(str[i]);
            		sdata = uniteMatrix.getColName(num[k]);
            		String sr[] = sdata.split(",");
            		sri = sr.length;
            		for(int j=0; j<sri; j++){
            			sd = Integer.parseInt(sr[j]);
            			data2 = ruimat.getCell(data1,sd);
            			if(data1 != sd){
            				if(min>data2){
            					min = data2;
            				
            				}
            			}
            		}
            	}
            	//融合に関係あるobjectをマトリクスに設定
         		umat.setCell(usize-2,count2,min);
         		umat.setCell(count2,usize-2,min);
         		data1 = 0;
         		data2 = 0;
         		count2++;
         		min = 1000000000;
            }
            
            //umatの配列名設定
            String[] ar = new String[usize-1];
            int z=0;
            for(int i=0; i<usize; i++){
            	if(i!=object1 && i!=object2){
            		ar[z] = uniteMatrix.getColName(i);
            		z++;
            	}
            }
            ar[usize-2] = st;
            umat.nameColCopy(ar);
            umat.nameRowCopy(ar);
            return umat;
          }
    
    
        /**
         * 特定の2つの対象の融合を行った時の類似マトリクスを返します。
         * @param object1 融合を行う1つ目の対象番号
         * @param object2 融合を行う2つ目の対象番号
         * @return 融合を行った類似マトリクス 
         */
             private ArithmeticMatrix getClinkObjectUniteMatrix(int object1, int object2, ArithmeticMatrix ruijiMatrix,ArithmeticMatrix uniteMatrix){
            	
             	int size = ruijiMatrix.getColLength();
                 //融合後のマトリクス
             	int usize = uniteMatrix.getColLength();
             	ArithmeticMatrix umat = new ArithmeticMatrix(usize-1,usize-1);
             	
             	//類似マトリクスを作成
             	ArithmeticMatrix ruimat = new ArithmeticMatrix();
             	ruimat.matrixCopy(ruijiMatrix);
             	//名前を取得する
             	String o1 = uniteMatrix.getColName(object1);
             	String o2 = uniteMatrix.getColName(object2);
             	String st = o1+o2;
             	String str[];
             	str = st.split(",");
             	int si = str.length;
             	
             	int[] num = new int[usize-1];
             	int count=0;
             	//融合させないオブジェクト番号を取得
             	for(int i=0; i<usize; i++){
             		if(i!=object1 && i!=object2){
             			num[count]=i;
             			count++;
             		}
             	}
             	//今回の融合には関係のないデータの配置
             	for(int j=0; j<usize-2; j++){
             		for(int k=0; k<usize-2; k++){
             			umat.setCell(j,k,uniteMatrix.getCell(num[j],num[k]));
             			umat.setCell(k,j,uniteMatrix.getCell(num[k],num[j]));
             		}
             	} 
                 int data1;
                 String sdata;
                 double data2;
                 int count2 = count;
                 count2 = 0;
                 int sri = 0;
                 int sd = 0;
                 double max = 0;
                 //今回の融合に関係のあるデータの配置
                 for(int k=0; k<count; k++){
                 	for(int i=0; i<si; i++){
                 		data1 = Integer.parseInt(str[i]);
                 		sdata = uniteMatrix.getColName(num[k]);
                 		String sr[] = sdata.split(",");
                 		sri = sr.length;
                 		for(int j=0; j<sri; j++){
                 			sd = Integer.parseInt(sr[j]);
                 			data2 = ruimat.getCell(data1,sd);
                 			if(data1 != sd){
                 				if(max<data2){
                 					max = data2;
                 				
                 				}
                 			}
                 		}
                 	}
                 	//融合に関係あるobjectをマトリクスに設定
              		umat.setCell(usize-2,count2,max);
              		umat.setCell(count2,usize-2,max);
              		data1 = 0;
              		data2 = 0;
              		count2++;
              		max = 0;
                 }
                 
                 //umatの配列名設定
                 String[] ar = new String[usize-1];
                 int z=0;
                 for(int i=0; i<usize; i++){
                 	if(i!=object1 && i!=object2){
                 		ar[z] = uniteMatrix.getColName(i);
                 		z++;
                 	}
                 }
                 ar[usize-2] = st;
                 umat.nameColCopy(ar);
                 umat.nameRowCopy(ar);
                 return umat;
               }
         
        
        
        
/**
* データマトリクスを設定します。
* @param baseMatrix 設定をするデータマトリクス
*/
    private void setMatrix(ArithmeticMatrix baseMatrix){
    	dataMatrix = new ArithmeticMatrix();
    	dataMatrix.matrixCopy(baseMatrix);
    }
    //最大値の行と列を返す。
    private int[] maxRowCol(ArithmeticMatrix mat){
    	int row =0;
    	int col =0;
    	int rowcol[] = new int[2];
    	double maxValue;
    	maxValue = mat.getCell(0,1);
    	row =0;
    	col =1;
    	for(int i=0; i<mat.getRowLength(); i++){
    		for(int j=0; j<mat.getColLength();j++){
    			if(i!=j){	
    				if(maxValue < mat.getCell(i,j)){
    					maxValue = mat.getCell(i,j);			
    					row = i;
    					col = j;
    				}
    			}
    		}
    	}
    	rowcol[0]= row;
    	rowcol[1]= col;
    	return rowcol;
    }
 
    //最小値の行と列を返す。
    private int[] minRowCol(ArithmeticMatrix mat){
    	int row = 0;
    	int col = 0;
    	int rowcol[] = new int[2];
    	double minValue;
    	minValue = mat.getCell(0,1);
    	row = 0;
    	col = 1;
    	for(int i=0; i<mat.getRowLength(); i++){
    		for(int j=0; j<mat.getColLength();j++){
    			if(i!=j){	
    				if(minValue > mat.getCell(i,j)){
    					minValue = mat.getCell(i,j);			
    					row = i;
    					col = j;
    				}
    			}
    		}
    	}
    
    	rowcol[0]= row;
    	rowcol[1]= col;
    	return rowcol;
    }
    
    /**
     * マトリクスの要素にある値から最小値を返します。
     * @param マトリクス
     * @return 要素内にある最小値
     */	
    	private double getMinCell(ArithmeticMatrix mat){
    		double minValue;
    		minValue = mat.getCell(0,1);
    		
    		for(int i=0; i<mat.getRowLength(); i++){
    			for(int j=0; j<mat.getColLength();j++){
    				if(i!=j){
    					if(minValue > mat.getCell(i,j)){
    						minValue = mat.getCell(i,j);				
    					}
    				}
    			}
    		}
    		return minValue;
    	}
    	
    	 /**
         * マトリクスの要素にある値から最小値を返します。
         * @param マトリクス
         * @return 要素内にある最小値
         */	
        	private double getMaxCell(ArithmeticMatrix mat){
        		double maxValue;
        		maxValue = mat.getCell(0,1);
        		
        		for(int i=0; i<mat.getRowLength(); i++){
        			for(int j=0; j<mat.getColLength();j++){
        				if(i!=j){
        					if(maxValue < mat.getCell(i,j)){
        						maxValue = mat.getCell(i,j);				
        					}
        				}
        			}
        		}
        		return maxValue;
        	}
    	
}

